@@ -1,14 +1,26 @@
+//! Auxiliary functions
+
+/// Check if `num` is a non‐zero power of two
 pub fn is_power_of_two(num: usize) -> bool {
 	num != 0 && (num & (num - 1)) == 0
 }
 
+/// Align `offset` to a multiple of `align`
+///
+/// `align` must be a non‐zero power of two.
 pub fn align(offset: usize, align: usize) -> usize {
 	debug_assert!(is_power_of_two(align));
 
 	(offset + (align - 1)) & !(align - 1)
 }
 
+/// Securely zero‐out memory
+///
+/// # Safety
+///
+/// `addr` must be [valid](std::ptr#safety) for writes and properly aligned.
 pub unsafe fn zero<T>(addr: *mut T, count: usize) {
+	debug_assert_eq!(addr.align_offset(std::mem::align_of::<T>()), 0);
 	std::intrinsics::volatile_set_memory(addr, 0, count);
 }
 
