@@ -2,6 +2,7 @@
 
 use crate::traits::Protectable;
 
+use std::intrinsics::likely;
 use std::fmt;
 use std::ops::{Deref, DerefMut, Index, IndexMut, Drop};
 use std::sync::atomic::{AtomicUsize, Ordering};
@@ -21,6 +22,7 @@ impl<T: Protectable> Guard<T> {
 	const MUT: usize = usize::MAX & Self::REF;
 	const MAX: usize = (usize::MAX & Self::REF) - 1;
 
+	#[inline]
 	pub fn from_inner(inner: T) -> Self {
 		Self(AtomicUsize::default(), inner)
 	}
@@ -65,7 +67,7 @@ impl<T: Protectable> Guard<T> {
 			Some(refs - 1)
 		}).unwrap() & Self::REF == 1 {  // Last release?
 		self.0.fetch_update(Ordering::AcqRel, Ordering::Acquire,
-		|refs| if refs == 1 {
+		|refs| if likely(refs == 1) {
 				// Last release
 				self.1.lock().unwrap();
 
@@ -96,10 +98,12 @@ impl<T: Protectable> Guard<T> {
 		self
 	}
 
+	#[inline]
 	pub(crate) unsafe fn inner(&self) -> &T {
 		&self.1
 	}
 
+	#[inline]
 	pub(crate) unsafe fn inner_mut(&mut self) -> &mut T {
 		&mut self.1
 	}
@@ -112,10 +116,12 @@ impl<T: Protectable> Guard<T> {
 		self
 	}
 
+	#[inline]
 	pub fn borrow(&self) -> Ref<'_, T> {
 		Ref(self.acquire())
 	}
 
+	#[inline]
 	pub fn borrow_mut(&mut self) -> RefMut<'_, T> {
 		RefMut(self.acquire_mut())
 	}
@@ -141,6 +147,7 @@ impl<T: Protectable> fmt::Debug for Guard<T> {
 }
 
 impl<T: Protectable> Ref<'_, T> {
+	#[inline]
 	pub fn inner(&self) -> &T {
 		&self.0.1
 	}
@@ -149,6 +156,7 @@ impl<T: Protectable> Ref<'_, T> {
 impl<T: Protectable + Deref> Deref for Ref<'_, T> {
 	type Target = T::Target;
 
+	#[inline]
 	fn deref(&self) -> &Self::Target {
 		&*self.0.1
 	}
@@ -157,12 +165,14 @@ impl<T: Protectable + Deref> Deref for Ref<'_, T> {
 impl<T: Protectable + Index<I>, I> Index<I> for Ref<'_, T> {
 	type Output = T::Output;
 
+	#[inline]
 	fn index(&self, index: I) -> &Self::Output {
 		&self.0.1[index]
 	}
 }
 
 impl<T: Protectable> Drop for Ref<'_, T> {
+	#[inline]
 	fn drop(&mut self) {
 		self.0.release();
 	}
@@ -175,10 +185,12 @@ impl<T: Protectable + fmt::Debug> fmt::Debug for Ref<'_, T> {
 }
 
 impl<T: Protectable> RefMut<'_, T> {
+	#[inline]
 	pub fn inner(&self) -> &T {
 		&self.0.1
 	}
 
+	#[inline]
 	pub fn inner_mut(&mut self) -> &mut T {
 		&mut self.0.1
 	}
@@ -187,6 +199,7 @@ impl<T: Protectable> RefMut<'_, T> {
 impl<T: Protectable + Deref> Deref for RefMut<'_, T> {
 	type Target = T::Target;
 
+	#[inline]
 	fn deref(&self) -> &Self::Target {
 		&*self.0.1
 	}
@@ -195,24 +208,28 @@ impl<T: Protectable + Deref> Deref for RefMut<'_, T> {
 impl<T: Protectable + Index<I>, I> Index<I> for RefMut<'_, T> {
 	type Output = T::Output;
 
+	#[inline]
 	fn index(&self, index: I) -> &Self::Output {
 		&self.0.1[index]
 	}
 }
 
 impl<T: Protectable + Index<I> + IndexMut<I>, I> IndexMut<I> for RefMut<'_, T> {
+	#[inline]
 	fn index_mut(&mut self, index: I) -> &mut Self::Output {
 		&mut self.0.1[index]
 	}
 }
 
 impl<T: Protectable + Deref + DerefMut> DerefMut for RefMut<'_, T> {
+	#[inline]
 	fn deref_mut(&mut self) -> &mut Self::Target {
 		&mut *self.0.1
 	}
 }
 
 impl<T: Protectable> Drop for RefMut<'_, T> {
+	#[inline]
 	fn drop(&mut self) {
 		self.0.release_mut();
 	}
@@ -223,3 +240,132 @@ impl<T: Protectable + fmt::Debug> fmt::Debug for RefMut<'_, T> {
 		fmt.debug_tuple("RefMut").field(&self.0).finish()
 	}
 }
+
+#[cfg(test)]
+mod tests {
+	use super::*;
+	use std::io::Error;
+
+	struct Dummy;
+
+	impl Protectable for Dummy {
+		fn lock(&self) -> Result<(), Error> {
+			Ok(())
+		}
+
+		fn unlock(&self) -> Result<(), Error> {
+			Ok(())
+		}
+
+		fn unlock_mut(&mut self) -> Result<(), Error> {
+			Ok(())
+		}
+	}
+
+	#[test]
+	#[should_panic]
+	fn underflow() {
+		let guard = Guard::from_inner(Dummy);
+		guard.release();
+	}
+
+	#[test]
+	#[should_panic]
+	fn overflow() {
+		let guard = Guard(AtomicUsize::new(Guard::<Dummy>::MAX), Dummy);
+		guard.acquire();
+	}
+
+	#[test]
+	fn immutable() {
+		let guard = Guard::from_inner(Dummy);
+
+		for _ in 0..1024 {
+			guard.acquire();
+		}
+
+		for _ in 0..1024 {
+			guard.release();
+		}
+	}
+
+	#[test]
+	fn mutable() {
+		let mut guard = Guard::from_inner(Dummy);
+
+		for _ in 0..1024 {
+			guard.acquire_mut();
+			guard.release_mut();
+		}
+	}
+
+	#[test]
+	#[should_panic]
+	fn mutable_multiple() {
+		let mut guard = Guard::from_inner(Dummy);
+		guard.acquire_mut();
+		guard.acquire_mut();
+	}
+
+	#[test]
+	fn borrow() {
+		const LIMIT: usize = 1024;
+
+		let guard = Guard::from_inner(Dummy);
+
+		{
+			let mut refs = std::vec::Vec::with_capacity(LIMIT);
+
+			for _ in 0..LIMIT {
+				refs.push(guard.borrow());
+			}
+		}
+
+		assert_eq!(guard.0.into_inner(), 0);
+	}
+
+	#[test]
+	fn borrow_mut() {
+		let mut guard = Guard::from_inner(Dummy);
+
+		{
+			guard.borrow_mut();
+		}
+
+		assert_eq!(guard.0.into_inner(), 0);
+	}
+
+	#[test]
+	fn concurrent() {
+		use std::cmp::max;
+		use std::sync::{Arc, Barrier};
+		use std::thread;
+
+		const LIMIT: usize = 262144;
+
+		let guard = Arc::new(Guard::from_inner(Dummy));
+		let concurrency = max(16, 2 * thread::available_parallelism().unwrap().get());
+		let barrier = Arc::new(Barrier::new(concurrency));
+		let mut threads = std::vec::Vec::with_capacity(concurrency);
+
+		for _ in 0..concurrency {
+			let barrier = barrier.clone();
+			let guard = guard.clone();
+
+			threads.push(thread::spawn(move || {
+				barrier.wait();
+
+				for _ in 0..LIMIT {
+					guard.borrow();
+					thread::yield_now();
+				}
+			}));
+		}
+
+		for thread in threads {
+			thread.join().unwrap();
+		}
+
+		assert_eq!(Arc::try_unwrap(guard).unwrap().0.into_inner(), 0);
+	}
+}
